package GII::Event::Key;

=head1 NAME

GII::Event::Key - Perl class encapsulating gii_key_event structures.

=head1 SYNOPSIS

    use GII::Event::Key;

    $ev = new GII::Event::Key;

    $mods = $ev->modifiers();
    $ev->modifiers($mods);

    $sym = $ev->sym();
    $ev->sym($sym);

    $label = $ev->label();
    $ev->label($label);

    $button = $ev->button();
    $ev->button($button);

    $key_down = $ev->press;
    $ev->press($key_down);

    $key_repeated = $ev->repeat;
    $ev->repeat($key_repeated);

    $key_up = $ev->release;
    $ev->release($key_up);

=head1 DESCRIPTION

The GII::Event::Key subclass provides simple methods to access
the extra members of a LibGII gii_key_event structure which cannot
be accessed via the methods in the class GII::Event.

=cut

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);
use GII::Event;

require Exporter;
require DynaLoader;
require AutoLoader;

@ISA = qw(Exporter AutoLoader DynaLoader GII::Event);

=head1 Exported Constants

NOTE: Before anyone yells at me for exporting these by default,
read the GII:: package documentation.  It is not normal usage
to explicitly use GII::Event::Key.  Normally, GII:: uses it and
the constants are thus available inside the GII:: package namespace.
Only use GII::Event::Key explicitly if you really want all the
keysym constants dumped into your namespace.

KT_LATIN1
KT_SPEC
KT_FN
KT_PAD
KT_MOD
KT_DEAD
KM_SHIFT
KM_CTRL
KM_ALT
KM_META
KM_SUPER
KM_HYPER
KM_ALTGR
KM_CAPS
KM_NUM
KM_SCROLL
KM_MASK
KM_RIGHT
KM_LOCK
MOD_SHIFT
MOD_CTRL
MOD_ALT
MOD_META
MOD_SUPER
MOD_HYPER
MOD_ALTGR
MOD_CAPS
MOD_NUM
MOD_SCROLL
K_DeadRing
K_DeadCaron
K_DeadOgonek
K_DeadIota
K_DeadDoubleAcute
K_DeadBreve
K_DeadAboveDot
K_DeadBelowDot
K_DeadVoicedSound
K_DeadSemiVoicedSound
K_DeadAcute
K_DeadCedilla
K_DeadCircumflex
K_DeadDiaeresis
K_DeadGrave
K_DeadTilde
K_DeadMacron
K_F0
K_F1
K_F2
K_F3
K_F4
K_F5
K_F6
K_F7
K_F8
K_F9
K_F10
K_F11
K_F12
K_F13
K_F14
K_F15
K_F16
K_F17
K_F18
K_F19
K_F20
K_F21
K_F22
K_F23
K_F24
K_F25
K_F26
K_F27
K_F28
K_F29
K_F30
K_F31
K_F32
K_F33
K_F34
K_F35
K_F36
K_F37
K_F38
K_F39
K_F40
K_F41
K_F42
K_F43
K_F44
K_F45
K_F46
K_F47
K_F48
K_F49
K_F50
K_F51
K_F52
K_F53
K_F54
K_F55
K_F56
K_F57
K_F58
K_F59
K_F60
K_F61
K_F62
K_F63
K_F64
K_VOID
K_Enter
K_Delete
K_Break
K_ScrollForw
K_ScrollBack
K_Boot
K_Compose
K_SAK
K_Undo
K_Redo
K_Menu
K_Cancel
K_PrintScreen
K_Execute
K_Find
K_Begin
K_Clear
K_Insert
K_Select
K_Macro
K_Help
K_Do
K_Pause
K_Stop
K_SysRq
K_ModeSwitch
K_Up
K_Down
K_Left
K_Right
K_Prior
K_PageUp
K_Next
K_PageDown
K_Home
K_End
K_P0
K_P1
K_P2
K_P3
K_P4
K_P5
K_P6
K_P7
K_P8
K_P9
K_PA
K_PB
K_PC
K_PD
K_PE
K_PF
K_PPlus
K_PMinus
K_PSlash
K_PAsterisk
K_PStar
K_PEqual
K_PSeparator
K_PDecimal
K_PParenLeft
K_PParenRight
K_PSpace
K_PEnter
K_PTab
K_PPlusMinus
K_PBegin
K_PF1
K_PF2
K_PF3
K_PF4
K_PF5
K_PF6
K_PF7
K_PF8
K_PF9
K_Shift
K_Ctrl
K_Alt
K_Meta
K_Super
K_Hyper
K_AltGr
K_Caps
K_Num
K_Scroll
K_ShiftL
K_ShiftR
K_CtrlL
K_CtrlR
K_AltL
K_AltR
K_MetaL
K_MetaR
K_SuperL
K_SuperR
K_HyperL
K_HyperR
K_ShiftLock
K_CtrlLock
K_AltLock
K_MetaLock
K_SuperLock
K_HyperLock
K_AltGrLock
K_CapsLock
K_NumLock
K_ScrollLock
K_NIL
UC_Nul
UC_BackSpace
UC_Tab
UC_Linefeed
UC_Return
UC_Escape
UC_Delete
UC_Space
UC_Exclamation
UC_Exclam
UC_DoubleQuote
UC_QuoteDbl
UC_NumberSign
UC_Hash
UC_Dollar
UC_Percent
UC_Ampersand
UC_Apostrophe
UC_ParenLeft
UC_ParenRight
UC_Asterisk
UC_Star
UC_Plus
UC_Comma
UC_Minus
UC_Period
UC_Slash
UC_0
UC_1
UC_2
UC_3
UC_4
UC_5
UC_6
UC_7
UC_8
UC_9
UC_Colon
UC_Semicolon
UC_Less
UC_Equal
UC_Greater
UC_Question
UC_At
UC_A
UC_B
UC_C
UC_D
UC_E
UC_F
UC_G
UC_H
UC_I
UC_J
UC_K
UC_L
UC_M
UC_N
UC_O
UC_P
UC_Q
UC_R
UC_S
UC_T
UC_U
UC_V
UC_W
UC_X
UC_Y
UC_Z
UC_BracketLeft
UC_BackSlash
UC_BracketRight
UC_Circumflex
UC_Hat
UC_Underscore
UC_Grave
UC_a
UC_b
UC_c
UC_d
UC_e
UC_f
UC_g
UC_h
UC_i
UC_j
UC_k
UC_l
UC_m
UC_n
UC_o
UC_p
UC_q
UC_r
UC_s
UC_t
UC_u
UC_v
UC_w
UC_x
UC_y
UC_z
UC_BraceLeft
UC_Bar
UC_Pipe
UC_BraceRight
UC_Tilde
UC_NoBreakSpace
UC_NBSP
UC_ExclamDown
UC_Cent
UC_Sterling
UC_Pound
UC_Currency
UC_Yen
UC_BrokenBar
UC_Section
UC_Diaeresis
UC_Umlaut
UC_Copyright
UC_OrdFeminine
UC_GuillemotLeft
UC_NotSign
UC_SoftHyphen
UC_Registered
UC_Macron
UC_Degree
UC_PlusMinus
UC_TwoSuperior
UC_ThreeSuperior
UC_Acute
UC_Mu
UC_Micro
UC_Paragraph
UC_Pilcrow
UC_PeriodCentered
UC_MiddleDot
UC_Cedilla
UC_OneSuperior
UC_mKuline
UC_GuillemotRight
UC_OneQuarter
UC_OneHalf
UC_ThreeQuarters
UC_QuestionDown
UC_Agrave
UC_Aacute
UC_Acircumflex
UC_Atilde
UC_Adiaeresis
UC_Aumlaut
UC_Aring
UC_AE
UC_Ccedilla
UC_Egrave
UC_Eacute
UC_Ecircumflex
UC_Ediaeresis
UC_Eumlaut
UC_Igrave
UC_Iacute
UC_Icircumflex
UC_Idiaeresis
UC_Iumlaut
UC_ETH
UC_Ntilde
UC_Ograve
UC_Oacute
UC_Ocircumflex
UC_Otilde
UC_Odiaeresis
UC_Oumlaut
UC_Multiply
UC_Ooblique
UC_Ugrave
UC_Uacute
UC_Ucircumflex
UC_Udiaeresis
UC_Uumlaut
UC_Yacute
UC_THORN
UC_ssharp
UC_agrave
UC_aacute
UC_acircumflex
UC_atilde
UC_adiaeresis
UC_aumlaut
UC_aring
UC_ae
UC_ccedilla
UC_egrave
UC_eacute
UC_ecircumflex
UC_ediaeresis
UC_eumlaut
UC_igrave
UC_iacute
UC_icircumflex
UC_idiaeresis
UC_iumlaut
UC_eth
UC_ntilde
UC_ograve
UC_oacute
UC_ocircumflex
UC_otilde
UC_odiaeresis
UC_oumlaut
UC_Division
UC_oslash
UC_ugrave
UC_uacute
UC_ucircumflex
UC_udiaeresis
UC_uumlaut
UC_yacute
UC_thorn
UC_ydiaeresis
UC_yumlaut
BUTTON_NIL

=cut

@EXPORT = qw(
KT_LATIN1
KT_SPEC
KT_FN
KT_PAD
KT_MOD
KT_DEAD
KM_SHIFT
KM_CTRL
KM_ALT
KM_META
KM_SUPER
KM_HYPER
KM_ALTGR
KM_CAPS
KM_NUM
KM_SCROLL
KM_MASK
KM_RIGHT
KM_LOCK
MOD_SHIFT
MOD_CTRL
MOD_ALT
MOD_META
MOD_SUPER
MOD_HYPER
MOD_ALTGR
MOD_CAPS
MOD_NUM
MOD_SCROLL
K_DeadRing
K_DeadCaron
K_DeadOgonek
K_DeadIota
K_DeadDoubleAcute
K_DeadBreve
K_DeadAboveDot
K_DeadBelowDot
K_DeadVoicedSound
K_DeadSemiVoicedSound
K_DeadAcute
K_DeadCedilla
K_DeadCircumflex
K_DeadDiaeresis
K_DeadGrave
K_DeadTilde
K_DeadMacron
K_F0
K_F1
K_F2
K_F3
K_F4
K_F5
K_F6
K_F7
K_F8
K_F9
K_F10
K_F11
K_F12
K_F13
K_F14
K_F15
K_F16
K_F17
K_F18
K_F19
K_F20
K_F21
K_F22
K_F23
K_F24
K_F25
K_F26
K_F27
K_F28
K_F29
K_F30
K_F31
K_F32
K_F33
K_F34
K_F35
K_F36
K_F37
K_F38
K_F39
K_F40
K_F41
K_F42
K_F43
K_F44
K_F45
K_F46
K_F47
K_F48
K_F49
K_F50
K_F51
K_F52
K_F53
K_F54
K_F55
K_F56
K_F57
K_F58
K_F59
K_F60
K_F61
K_F62
K_F63
K_F64
K_VOID
K_Enter
K_Delete
K_Break
K_ScrollForw
K_ScrollBack
K_Boot
K_Compose
K_SAK
K_Undo
K_Redo
K_Menu
K_Cancel
K_PrintScreen
K_Execute
K_Find
K_Begin
K_Clear
K_Insert
K_Select
K_Macro
K_Help
K_Do
K_Pause
K_Stop
K_SysRq
K_ModeSwitch
K_Up
K_Down
K_Left
K_Right
K_Prior
K_PageUp
K_Next
K_PageDown
K_Home
K_End
K_P0
K_P1
K_P2
K_P3
K_P4
K_P5
K_P6
K_P7
K_P8
K_P9
K_PA
K_PB
K_PC
K_PD
K_PE
K_PF
K_PPlus
K_PMinus
K_PSlash
K_PAsterisk
K_PStar
K_PEqual
K_PSeparator
K_PDecimal
K_PParenLeft
K_PParenRight
K_PSpace
K_PEnter
K_PTab
K_PPlusMinus
K_PBegin
K_PF1
K_PF2
K_PF3
K_PF4
K_PF5
K_PF6
K_PF7
K_PF8
K_PF9
K_Shift
K_Ctrl
K_Alt
K_Meta
K_Super
K_Hyper
K_AltGr
K_Caps
K_Num
K_Scroll
K_ShiftL
K_ShiftR
K_CtrlL
K_CtrlR
K_AltL
K_AltR
K_MetaL
K_MetaR
K_SuperL
K_SuperR
K_HyperL
K_HyperR
K_ShiftLock
K_CtrlLock
K_AltLock
K_MetaLock
K_SuperLock
K_HyperLock
K_AltGrLock
K_CapsLock
K_NumLock
K_ScrollLock
K_NIL
UC_Nul
UC_BackSpace
UC_Tab
UC_Linefeed
UC_Return
UC_Escape
UC_Delete
UC_Space
UC_Exclamation
UC_Exclam
UC_DoubleQuote
UC_QuoteDbl
UC_NumberSign
UC_Hash
UC_Dollar
UC_Percent
UC_Ampersand
UC_Apostrophe
UC_ParenLeft
UC_ParenRight
UC_Asterisk
UC_Star
UC_Plus
UC_Comma
UC_Minus
UC_Period
UC_Slash
UC_0
UC_1
UC_2
UC_3
UC_4
UC_5
UC_6
UC_7
UC_8
UC_9
UC_Colon
UC_Semicolon
UC_Less
UC_Equal
UC_Greater
UC_Question
UC_At
UC_A
UC_B
UC_C
UC_D
UC_E
UC_F
UC_G
UC_H
UC_I
UC_J
UC_K
UC_L
UC_M
UC_N
UC_O
UC_P
UC_Q
UC_R
UC_S
UC_T
UC_U
UC_V
UC_W
UC_X
UC_Y
UC_Z
UC_BracketLeft
UC_BackSlash
UC_BracketRight
UC_Circumflex
UC_Hat
UC_Underscore
UC_Grave
UC_a
UC_b
UC_c
UC_d
UC_e
UC_f
UC_g
UC_h
UC_i
UC_j
UC_k
UC_l
UC_m
UC_n
UC_o
UC_p
UC_q
UC_r
UC_s
UC_t
UC_u
UC_v
UC_w
UC_x
UC_y
UC_z
UC_BraceLeft
UC_Bar
UC_Pipe
UC_BraceRight
UC_Tilde
UC_NoBreakSpace
UC_NBSP
UC_ExclamDown
UC_Cent
UC_Sterling
UC_Pound
UC_Currency
UC_Yen
UC_BrokenBar
UC_Section
UC_Diaeresis
UC_Umlaut
UC_Copyright
UC_OrdFeminine
UC_GuillemotLeft
UC_NotSign
UC_SoftHyphen
UC_Registered
UC_Macron
UC_Degree
UC_PlusMinus
UC_TwoSuperior
UC_ThreeSuperior
UC_Acute
UC_Mu
UC_Micro
UC_Paragraph
UC_Pilcrow
UC_PeriodCentered
UC_MiddleDot
UC_Cedilla
UC_OneSuperior
UC_mKuline
UC_GuillemotRight
UC_OneQuarter
UC_OneHalf
UC_ThreeQuarters
UC_QuestionDown
UC_Agrave
UC_Aacute
UC_Acircumflex
UC_Atilde
UC_Adiaeresis
UC_Aumlaut
UC_Aring
UC_AE
UC_Ccedilla
UC_Egrave
UC_Eacute
UC_Ecircumflex
UC_Ediaeresis
UC_Eumlaut
UC_Igrave
UC_Iacute
UC_Icircumflex
UC_Idiaeresis
UC_Iumlaut
UC_ETH
UC_Ntilde
UC_Ograve
UC_Oacute
UC_Ocircumflex
UC_Otilde
UC_Odiaeresis
UC_Oumlaut
UC_Multiply
UC_Ooblique
UC_Ugrave
UC_Uacute
UC_Ucircumflex
UC_Udiaeresis
UC_Uumlaut
UC_Yacute
UC_THORN
UC_ssharp
UC_agrave
UC_aacute
UC_acircumflex
UC_atilde
UC_adiaeresis
UC_aumlaut
UC_aring
UC_ae
UC_ccedilla
UC_egrave
UC_eacute
UC_ecircumflex
UC_ediaeresis
UC_eumlaut
UC_igrave
UC_iacute
UC_icircumflex
UC_idiaeresis
UC_iumlaut
UC_eth
UC_ntilde
UC_ograve
UC_oacute
UC_ocircumflex
UC_otilde
UC_odiaeresis
UC_oumlaut
UC_Division
UC_oslash
UC_ugrave
UC_uacute
UC_ucircumflex
UC_udiaeresis
UC_uumlaut
UC_yacute
UC_thorn
UC_ydiaeresis
UC_yumlaut
BUTTON_NIL
);
$VERSION = '0.01';

bootstrap GII::Event::Key $VERSION;

=head1 Methods

The extra methods and differences between GII::Event methods and the
ones provided in the GII::Event::Key class are as follows:

=head2 new

The GII::Event::Key constructor takes all the same arguments
as the GII::Event constructor, and treats them the same with the
exception that it does not do any automagical subclassing.  All
calls to this constructor will either fail or allocate/reuse
a gii_key_event structure.

Since this event is generated by the environment, this constructor
will be used only very rarely for debugging purposes.

=cut

sub new {
    my($class) = shift();
    my($type) = shift();
    my($this) = {};
    bless $this, $class;
    my($obj) = shift();
    if ($obj) {
        $this->{obj} = $obj;
        $this->set_type($obj, $type) if defined($type);
        $this->set_size($obj, $this->default_size) if defined($type);
        return($this);
    }
    $this->{obj} = $this->_alloc;
    $this->set_size($this->{obj}, $this->default_size);
    $this->set_type($this->{obj}, $type);
    return($this);
}

=head2 modifiers

Without an argument, returns the modifiers in effect when 
the key changed state.  With an argument, sets said value,
returning the value which was set.

=cut

sub modifiers {
    my($this) = shift();
    if (@_ + 0) {
        return $this->set_modifiers($this->{obj}, shift());
    }
    return $this->get_modifiers($this->{obj});
}

=head2 sym

Without an argument, returns the keysym of the key
which has changed state.  With an argument, sets said value,
returning the value which was set.

=cut

sub sym {
    my($this) = shift();
    if (@_ + 0) {
        return $this->set_sym($this->{obj}, shift());
    }
    return $this->get_sym($this->{obj});
}

=head2 label

Without an argument, returns the label of the key
which has changed state.  With an argument, sets said value,
returning the value which was set.

=cut

sub label {
    my($this) = shift();
    if (@_ + 0) {
        return $this->set_label($this->{obj}, shift());
    }
    return $this->get_label($this->{obj});
}

=head2 button

Without an argument, returns the button number of the key
which has changed state.  With an argument, sets said value,
returning the value which was set.

=cut

sub button {
    my($this) = shift();
    if (@_ + 0) {
        return $this->set_button($this->{obj}, shift());
    }
    return $this->get_button($this->{obj});
}

=head2 press

Without an argument, returns 0 if the event is a key release,
1 if the event is a key press, and 2 if the event is an automatic 
key repeat.  If the type is invalid for this event class,
returns undef.

With an argument, sets the event to a key press if the argument
is true in a boolean context, else to a key release, and returns 
the LibGII gii_event_type which was set.

=cut

sub press {
    my($this) = shift();
    if (@_ + 0) {
        if (shift()) {
            $this->set_type($this->{obj}, &evKeyPress);
            return &evKeyPress;
        }
        else {
            $this->set_type($this->{obj}, &evKeyRelease);
            return &evKeyRelease;
        }
    }
    return 1 if $this->get_type($this->{obj}) == &evKeyPress;
    return 0 if $this->get_type($this->{obj}) == &evKeyRelease;
    return 2 if $this->get_type($this->{obj}) == &evKeyRepeat;
    return undef;
}

=head2 release

Without an argument, returns 1 if the event is a key release,
0 if the event is a key press or a key repeat.  If the type is 
invalid for this event class, returns undef.

With an argument, sets the event to a key release if the argument
is true in a boolean context, else to a key press event, 
and returns the LibGII gii_event_type which was set.

=cut

sub release {
    my($this) = shift();
    if (@_ + 0) {
        if (shift()) {
            $this->set_type($this->{obj}, &evKeyRelease);
            return &evKeyRelease;
        }
        else {
            $this->set_type($this->{obj}, &evKeyPress);
            return &evKeyPress;
        }
    }

    return 1 if $this->get_type($this->{obj}) == &evKeyRelease;
    return 0 if $this->get_type($this->{obj}) == &evKeyPress;
    return 0 if $this->get_type($this->{obj}) == &evKeyRepeat;
    return undef;
}

=head2 repeat

Without an argument, returns 1 if the event is a key repeat,
0 if the event is a key press or release.  If the type is 
invalid for this event class, returns undef.

With an argument, sets the event to a key repeat if the argument
is true in a boolean context, else to a key release, and returns 
the LibGII gii_event_type which was set.

=cut

sub repeat {
    my($this) = shift();
    if (@_ + 0) {
        if (shift()) {
            $this->set_type($this->{obj}, &evKeyRepeat);
            return &evKeyRepeat;
        }
        else {
            $this->set_type($this->{obj}, &evKeyRelease);
            return &evKeyRelease;
        }
    }
    return 1 if $this->get_type($this->{obj}) == &evKeyRepeat;
    return 0 if $this->get_type($this->{obj}) == &evKeyPress;
    return 0 if $this->get_type($this->{obj}) == &evKeyRelease;
    return undef;
}



1;
__END__

=head1 AUTHOR

Brian S. Julin <bri@calyx.com>

=head1 SEE ALSO

GII::Event(3pm), GII::(3pm), perl(1).

=cut
